/*-------------------------------------------------------------------
 *
 *  @file names.c
 *
 *  @author Collin R. Mulliner <collin@mulliner.org>
 *
 *  This is part of the Bluetooth Location Tracker (BLT) project.
 *
 *-------------------------------------------------------------------*/

#include "bltwebd.h"

#ifdef DN_TREE
int blt_dn_walkTree(struct tnode_t *t, FILE *fp, int WorF)
{
	char buffer[1024];
	

	if (t->r != NULL) blt_dn_walkTree(t->r, fp, WorF);
	if (t->l != NULL) blt_dn_walkTree(t->l, fp, WorF);

	if (WorF) {
		memset(buffer, 0, 1024);
		printWithEsc(t->dev.dev.name, buffer, 1024);
		
		fprintf(fp, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X \"%s\"\n",
			t->dev.dev.bdAddr[0],t->dev.dev.bdAddr[1],t->dev.dev.bdAddr[2],
			t->dev.dev.bdAddr[3],t->dev.dev.bdAddr[4],t->dev.dev.bdAddr[5],
			buffer);
	}
	else {
		if (t->r != NULL) free(t->r);
		if (t->l != NULL) free(t->l);
	}
	return(1);
}

void blt_dn_init(struct blt_dev_names_t *dn)
{
	dn->numDevices = 0;
	dn->devices = NULL;
}

void blt_dn_destroy(struct blt_dev_names_t *dn)
{
	if (dn->devices != NULL) {
		blt_dn_walkTree(dn->devices, NULL, 0);
		free(dn->devices);
		dn->devices = NULL;
	}
}

int blt_dn_write(struct blt_dev_names_t *dn, struct blt_device_status_t *nDev)
{
	struct tnode_t *tree = dn->devices;
	int pos = 0;
	
	
	/* empty tree */	
	if (dn->devices == NULL) {
		dn->devices = malloc(sizeof(struct tnode_t));
		dn->devices->l = NULL;
		dn->devices->r = NULL;
		memcpy(&dn->devices->dev, nDev, sizeof(struct blt_device_status_t));
		dn->numDevices++;
				
		#ifdef DEBUG3
		fprintf(stderr, "new device: %.2X:%.2X:%.2X:%.2X:%.2X:%.2X name: %s\n",
			nDev->dev.bdAddr[0],nDev->dev.bdAddr[1],nDev->dev.bdAddr[2],nDev->dev.bdAddr[3],
			nDev->dev.bdAddr[4],nDev->dev.bdAddr[5], nDev->dev.name);
		#endif
		
		return(1);
	}

	do {
		pos = memcmp(tree->dev.dev.bdAddr, nDev->dev.bdAddr, 6);
		
		/* update */
		if (pos == 0) {		
			#ifdef DEBUG3
			fprintf(stderr, "update device: %.2X:%.2X:%.2X:%.2X:%.2X:%.2X name: %s\n",
				nDev->dev.bdAddr[0],nDev->dev.bdAddr[1],nDev->dev.bdAddr[2],nDev->dev.bdAddr[3],
				nDev->dev.bdAddr[4],nDev->dev.bdAddr[5], nDev->dev.name);
			#endif
			memcpy(&tree->dev, nDev, sizeof(struct blt_device_status_t));
			break;
		}
		/* left */
		else if (pos < 0) {
			if (tree->l != NULL) {
				tree = tree->l;
				continue;
			}
			else {
				tree->l = malloc(sizeof(struct tnode_t));
				tree = tree->l;
			}
		}
		/* right */
		else if (pos > 0) {
			if (tree->r != NULL) {
				tree = tree->r;
				continue;
			}
			else {
				tree->r = malloc(sizeof(struct tnode_t));
				tree = tree->r;
			}
		}

		/* insert */
		#ifdef DEBUG3
		fprintf(stderr, "new device: %.2X:%.2X:%.2X:%.2X:%.2X:%.2X name: %s\n",
			nDev->dev.bdAddr[0],nDev->dev.bdAddr[1],nDev->dev.bdAddr[2],nDev->dev.bdAddr[3],
			nDev->dev.bdAddr[4],nDev->dev.bdAddr[5], nDev->dev.name);
		#endif
		dn->numDevices++;
		tree->l = NULL;
		tree->r = NULL;
		memcpy(&tree->dev, nDev, sizeof(struct blt_device_status_t));
		break;
		
	} while(1);
		
	return(1);
}
#endif

#ifdef DN_HASH
void blt_dn_init(struct blt_dev_names_t *dn)
{
	dn->maxNumDevices = DN_HASHTABLE_SIZE;
	dn->numDevices = 0;
	dn->devices = malloc(dn->maxNumDevices * sizeof(struct blt_device_status_t));
	memset(dn->devices, 0, dn->maxNumDevices * sizeof(struct blt_device_status_t));
}

void blt_dn_destroy(struct blt_dev_names_t *dn)
{
	if (dn->devices != NULL) free(dn->devices);
}

int hash(unsigned char *mac, int tableSize)
{
	unsigned int *value;
	value = (unsigned int*) (mac+2);	
	return((*value) % tableSize);
}

int blt_dn_write(struct blt_dev_names_t *dn, struct blt_device_status_t *nDev)
{
	const unsigned char EMPTY_MAC[6] = { 0x00,0x00,0x00,0x00,0x00,0x00 };
	int pos;
	int bong;
	
		
	pos = hash(nDev->dev.bdAddr, dn->maxNumDevices);
	do {
		/* update */
		if (memcmp(dn->devices[pos].dev.bdAddr, nDev->dev.bdAddr, 6) == 0) {
			
			/* only update if newer! */
			if (nDev->tStamp >= dn->devices[pos].tStamp) {
		
				#ifdef DEBUG3
				fprintf(stderr, "updated device: %.2X:%.2X:%.2X:%.2X:%.2X:%.2X name: %s\n",
					nDev->dev.bdAddr[0],nDev->dev.bdAddr[1],nDev->dev.bdAddr[2],nDev->dev.bdAddr[3],nDev->dev.bdAddr[4],nDev->dev.bdAddr[5],
					nDev->dev.name);
				#endif
			
				memcpy(&dn->devices[pos], nDev, sizeof(struct blt_device_status_t));
			}
			else {
				return(0);
			}
			break;
		}
		/* insert */
		else if (memcmp(dn->devices[pos].dev.bdAddr, EMPTY_MAC, 6) == 0) {
			#ifdef DEBUG3
			fprintf(stderr, "updated device: %.2X:%.2X:%.2X:%.2X:%.2X:%.2X name: %s\n",
				nDev->dev.bdAddr[0],nDev->dev.bdAddr[1],nDev->dev.bdAddr[2],nDev->dev.bdAddr[3],nDev->dev.bdAddr[4],nDev->dev.bdAddr[5],
				nDev->dev.name);
			#endif
			dn->numDevices++;
			memcpy(&dn->devices[pos], nDev, sizeof(struct blt_device_status_t));
			break;
		}
		/* collision */
		else {
			pos++;
			/* if table end is reached wrap to begin of table */
			if (pos == dn->maxNumDevices) {
				pos = 0;
				if (bong == 0) bong = 1;
				else {
					fprintf(stderr, "ERROR: hashtable full!\n");
					return(0);
				}
			}
		}
	} while (1);
	
	return(1);
}

int blt_dn_read(struct blt_dev_names_t *dn, struct blt_device_status_t *nDev)
{
	const unsigned char EMPTY_MAC[6] = { 0x00,0x00,0x00,0x00,0x00,0x00 };
	int pos;
	int bong = 0;


	pos = hash(nDev->dev.bdAddr, dn->maxNumDevices);
	do {
		if (memcmp(dn->devices[pos].dev.bdAddr, nDev->dev.bdAddr, 6) == 0) {
			memcpy(nDev, &dn->devices[pos], sizeof(struct blt_device_status_t));
			break;
		}
		else if (memcmp(dn->devices[pos].dev.bdAddr, EMPTY_MAC, 6) == 0) {
			return(0);
		}
		else {
			pos++;
			/* if table end is reached wrap to begin of table */
			if (pos == dn->maxNumDevices) {
				pos = 0;
				if (bong == 0) bong = 1;
				else {
					fprintf(stderr, "ERROR: hashtable full!\n");
					return(0);
				}
			}
		}
	} while(1);
	
	return(1);
}
#endif
